@@ -5,7 +5,6 @@ use std::path::PathBuf;
 
 pub async fn run(settings: &Settings) -> Result<()> {
     let session_path = settings.session_path.as_str();
-    let key_path = settings.key_path.as_str();
 
     if !PathBuf::from(session_path).exists() {
         bail!("You are not logged in");
@@ -25,10 +24,6 @@ pub async fn run(settings: &Settings) -> Result<()> {
         remove_file(PathBuf::from(session_path))?;
     }
 
-    if PathBuf::from(key_path).exists() {
-        remove_file(PathBuf::from(key_path))?;
-    }
-
     println!("Your account is deleted");
 
     Ok(())
