@@ -1,21 +1,22 @@
 use std::collections::{HashMap, HashSet};
 
+use atuin_common::utils::Escapable as _;
 use clap::Parser;
 use crossterm::style::{Color, ResetColor, SetAttribute, SetForegroundColor};
-use eyre::{bail, Result};
+use eyre::Result;
 use interim::parse_date_string;
 
 use atuin_client::{
     database::{current_context, Database},
     history::History,
-    settings::{FilterMode, Settings},
+    settings::Settings,
 };
 use time::{Duration, OffsetDateTime, Time};
 
 #[derive(Parser, Debug)]
 #[command(infer_subcommands = true)]
 pub struct Cmd {
-    /// compute statistics for the specified period, leave blank for statistics since the beginning
+    /// Compute statistics for the specified period, leave blank for statistics since the beginning. See https://docs.atuin.sh/reference/stats/ for more details.
     period: Vec<String>,
 
     /// How many top commands to list
@@ -23,7 +24,7 @@ pub struct Cmd {
     count: usize,
 }
 
-fn compute_stats(settings: &Settings, history: &[History], count: usize) -> Result<()> {
+fn compute_stats(settings: &Settings, history: &[History], count: usize) {
     let mut commands = HashSet::<&str>::with_capacity(history.len());
     let mut prefixes = HashMap::<&str, usize>::with_capacity(history.len());
     for i in history {
@@ -40,7 +41,8 @@ fn compute_stats(settings: &Settings, history: &[History], count: usize) -> Resu
     top.sort_unstable_by_key(|x| std::cmp::Reverse(x.1));
     top.truncate(count);
     if top.is_empty() {
-        bail!("No commands found");
+        println!("No commands found");
+        return;
     }
 
     let max = top.iter().map(|x| x.1).max().unwrap();
@@ -66,12 +68,13 @@ fn compute_stats(settings: &Settings, history: &[History], count: usize) -> Resu
             print!(" ");
         }
 
-        println!("{ResetColor}] {gray}{count:num_pad$}{ResetColor} {bold}{command}{ResetColor}");
+        println!(
+            "{ResetColor}] {gray}{count:num_pad$}{ResetColor} {bold}{}{ResetColor}",
+            command.escape_control()
+        );
     }
     println!("Total commands:   {}", history.len());
     println!("Unique commands:  {unique}");
-
-    Ok(())
 }
 
 impl Cmd {
@@ -83,13 +86,11 @@ impl Cmd {
             self.period.join(" ")
         };
 
-        let now = OffsetDateTime::now_utc();
-        let now = settings.local_tz.map_or(now, |local| now.to_offset(local));
+        let now = OffsetDateTime::now_utc().to_offset(settings.timezone.0);
         let last_night = now.replace_time(Time::MIDNIGHT);
 
         let history = if words.as_str() == "all" {
-            db.list(FilterMode::Global, &context, None, false, false)
-                .await?
+            db.list(&[], &context, None, false, false).await?
         } else if words.trim() == "today" {
             let start = last_night;
             let end = start + Duration::days(1);
@@ -111,7 +112,7 @@ impl Cmd {
             let end = start + Duration::days(1);
             db.range(start, end).await?
         };
-        compute_stats(settings, &history, self.count)?;
+        compute_stats(settings, &history, self.count);
         Ok(())
     }
 }
@@ -181,7 +182,7 @@ mod tests {
 
     #[test]
     fn interesting_commands() {
-        let settings = Settings::default();
+        let settings = Settings::utc();
 
         assert_eq!(interesting_command(&settings, "cargo"), "cargo");
         assert_eq!(
@@ -198,7 +199,7 @@ mod tests {
     // Test with spaces in the common_prefix
     #[test]
     fn interesting_commands_spaces() {
-        let mut settings = Settings::default();
+        let mut settings = Settings::utc();
         settings.stats.common_prefix.push("sudo test".to_string());
 
         assert_eq!(interesting_command(&settings, "sudo test"), "sudo test");
@@ -223,7 +224,7 @@ mod tests {
     // Test with spaces in the common_subcommand
     #[test]
     fn interesting_commands_spaces_subcommand() {
-        let mut settings = Settings::default();
+        let mut settings = Settings::utc();
         settings
             .stats
             .common_subcommands
@@ -253,7 +254,7 @@ mod tests {
     // Test with spaces in the common_prefix and common_subcommand
     #[test]
     fn interesting_commands_spaces_both() {
-        let mut settings = Settings::default();
+        let mut settings = Settings::utc();
         settings.stats.common_prefix.push("sudo test".to_string());
         settings
             .stats
