@@ -2,6 +2,9 @@ use clap::{CommandFactory, Subcommand};
 use clap_complete::{generate, generate_to, Shell};
 use eyre::Result;
 
+#[cfg(not(windows))]
+use rustix::{fs::Mode, process::umask};
+
 #[cfg(feature = "client")]
 mod client;
 
@@ -46,6 +49,14 @@ pub enum AtuinCmd {
 
 impl AtuinCmd {
     pub fn run(self) -> Result<()> {
+        #[cfg(not(windows))]
+        {
+            // set umask before we potentially open/create files
+            // or in other words, 077. Do not allow any access to any other user
+            let mode = Mode::RWXG | Mode::RWXO;
+            umask(mode);
+        }
+
         match self {
             #[cfg(feature = "client")]
             Self::Client(client) => client.run(),
